# -*- coding: utf-8 -*
import struct

def print_hex(bytes):
    l = [hex(int(i)) for i in bytes]
    print("hex:"," ".join(l))

def long2bits(val):
    return [val&0x7f,(val>>7)&0x7f,(val>>14)&0x7f,(val>>21)&0x7f,(val>>28)&0x7f]

def ushort2bits(val):
    return [val&0x7f,(val>>7)&0x7f]

def short2bits(val):
    return [val&0x7f,(val>>7)&0x7f,(val>>14)&0x7f]

def float2bits(val):
    data = float2bytes(val)
    lval = bytes2long(data)
    bits = long2bits(lval)
    return bits

def int2bits(val):
    val = int2uint(val)
    return ushort2bits(val)

def bits2float(bits):
    if(len(bits)==5):
        val = 0
        for i in range(5):
            val+=bits[i]<<(i*7)
        v = [val&0xff,(val>>8)&0xff,(val>>16)&0xff,(val>>24)&0xff]
        return struct.unpack('<f', struct.pack('4B', *v))[0]
    return 0

def bits2short(bits):
    if(len(bits)==2):
        val = 0
        for i in range(2):
            val+=bits[i]<<(i*7)
        return val
    return 0

def bits2long(bits):
    if(len(bits)==5):
        val = 0
        for i in range(5):
            val+=bits[i]<<(i*7)
        return val
    return 0

def int2uint(val):
    if val<0:
        return val+256
    return val

def bits2int8(bits):
    val = 0
    for i in range(2):
        val+=bits[i]<<(i*7)
    if val>127:
        return val-256
    return val

def bytes2long(buf,position=0):
    v = [buf[position], buf[position+1], buf[position+2], buf[position+3]]
    return struct.unpack('<l', struct.pack('4B', *v))[0]

def bytes2short(buf, position=0):
    v = [buf[position], buf[position+1]]
    return struct.unpack('<h', struct.pack('2B', *v))[0]
    
def bytes2float(buf, position=0):
    v = [buf[position], buf[position+1],buf[position+2],buf[position+3]]
    return struct.unpack('<f', struct.pack('4B', *v))[0]

def float2bytes(fval):
    val = struct.pack("f",fval)
    return [val[0],val[1],val[2],val[3]]

def long2bytes(lval):
    val = struct.pack("l",lval)
    return [val[0],val[1],val[2],val[3]]

def short2bytes(sval):
    val = struct.pack("h",sval)
    return [val[0],val[1]]

def ushort2bytes(sval):
    val = struct.pack("H",sval)
    return [val[0],val[1]]

def string2bytes(ssval):
    return [ord(i) for i in ssval]

def bytes2string(buf):
    return "".join([ chr(i) for i in buf])


# -*- coding: utf-8 -*
import os
import threading
from time import  sleep
import serial.tools.list_ports
import serial
import mkpiano

def connect(port,baudrate=115200):
    """
    .. code-block:: python
        :linenos:

        from mkpiano import SerialPort
        from mkpiano import MegaPi

        uart = SerialPort.connect("COM3")
        board = MegaPi.connect(uart)

    """
    uart = SerialPort(port,baudrate)
    return uart

create = connect

class SerialPort():
    """
    """
    def __init__(self, port="/dev/ttyAMA0", baudrate=115200, timeout=1):
        self.exiting = False
        self._is_sending = True
        self._responses = []
        self._queue = []
        self._ser = None
        try:
            self._ser = serial.Serial(port,baudrate)
            self._ser.timeout = 0.01
            sleep(1)
            self._thread = threading.Thread(target=self._on_read,args=(self._callback,))
            self._thread.daemon = True
            self._thread.start()
            self._exit_thread = threading.Thread(target=self._on_exiting,args=())
            self._exit_thread.daemon = True
            self._exit_thread.start()
            mkpiano.add_port(self)
        except Exception as ex:
            print('$#&*@{"err_code":101,"err_msg":"串口被占用","device":"mkpiano","extra":"'+str(ex)+'"}@*&#$')

    def setup(self,callback):
        self._responses.append(callback)

    @property
    def type(self):
        return "uart"
        
    def _callback(self,received):
        for method in self._responses:
            method(received)

    def _on_read(self,callback):
        while True:
            if self.exiting:
                break
            if self._is_sending:
                self.__sending()
                    
            if self.is_open():
                # if self.in_waiting()>0:
                buf = self.read()
                for i in range(len(buf)):
                    callback(buf[i])
            sleep(0.001)

    def send(self,buffer):
        if self.is_open():
            self._queue.append(buffer)
        # sleep(0.002)

    def __sending(self):
        if len(self._queue)>0:
            if self.is_open():
                buf = self._queue[0]
                try:
                    self._ser.write(buf)
                except serial.serialutil.SerialException as e:
                    self.exiting = True
                    print("\033[1;33m连接失败，未检测到设备\033[0m")
                    print('$#&*@{"err_code":102,"err_msg":"发送数据失败，未检测到设备","device":"mkpiano","extra":{}}@*&#$')
                    return None
                self._queue.pop(0)

    def read(self):
        try:
            return self._ser.read(self.in_waiting())
        except serial.serialutil.SerialException as e:
            self.exiting = True
            print("\033[1;33m连接失败，未检测到设备\033[0m")
            print('$#&*@{"err_code":102,"err_msg":"读取数据失败，未检测到设备","device":"mkpiano","extra":{}}@*&#$')
            return []

    def enable_sending(self):
        self._is_sending = True

    def disable_sending(self):
        self._is_sending = False
        
    def _on_exiting(self):
        while True:
            if self.exiting:
                self.exit()
                break
            sleep(0.001)
    def is_open(self):
        if not self._ser is None:
            return self._ser.isOpen()
        return False

    def in_waiting(self):
        return self._ser.inWaiting()

    def close(self):
        self._ser.close()

    def exit(self):
        if not self._thread is None:
            self._is_sending = False
            self.exiting = True
            self._thread.join()
            self._thread = None
            self.close()
            os._exit(0)

    @staticmethod
    def list():
        """
        获取串口列表

        .. code-block:: python
            :linenos:

            from mkpiano import SerialPort
            print(SerialPort.list())

        :param: 无
        :return: 串口列表
        """
        return serial.tools.list_ports.comports()



from .boards import mbuild
import sys
import signal
import os,platform,ctypes,subprocess
from time import sleep
def driver_exist():
    system = platform.system()
    if system=="Darwin":
        kexts = "_".join(os.listdir("/Library/Extensions/"))
        return "usbserial.kext" in kexts
    elif system=="Windows":
        return "ch341ser.inf" in "_".join(os.listdir("C:\Windows\System32\DriverStore\FileRepository"))
    return True

def driver_install():
    if driver_exist():
        return True
    system = platform.system()
    try:
        if system=="Darwin":
            subprocess.call(["/usr/bin/open",os.path.dirname(__file__)+"/driver/ch34x_install.pkg"])
        elif system=="Windows":
            os.chdir(os.path.dirname(__file__)+"\\driver\\")
            if ctypes.windll.shell32.IsUserAnAdmin():
                subprocess.call([os.path.dirname(__file__)+"\\driver\\ch34x_install.exe","/sw","/se"])
            else:
                ctypes.windll.shell32.ShellExecuteW(None, "open", os.path.dirname(__file__)+"\\driver\\ch34x_install.exe", None, None, 1)
        while not driver_exist():
            sleep(1)
        return True
    except Exception as ex:
        pass
    return False

if driver_exist() == False:
    print("\033[1;32m设备驱动安装中...\033[0m")
    if driver_install() == False:
        print("\033[1;33m驱动安装失败\033[0m")
        print('$#&*@{"err_code":104,"err_msg":"驱动安装失败","device":"mkpiano","extra":{}}@*&#$')
        
_ports = []
_threads = []

def add_port(port):
    global _ports
    _ports.append(port)

def add_thread(thread):
    global _threads
    thread.daemon = True
    _threads.append(thread)

def quit():
    __exiting(0,0)
    
def __exiting(signal, frame):
    global _ports
    global _threads
    for port in _ports:
        port.exit()
    for thread in _threads:
        thread.exit()
    try:
        sys.exit(0)
    except Exception as e:
        print(e)

signal.signal(signal.SIGINT, __exiting)
piano = mbuild.piano
sleep(0.5)


# -*- coding: utf-8 -*
from ...protocols import HalocodeProtocol,MegaPiProtocol,NeuronProtocol
from ...protocols import HalocodePackData
from ...comm.SerialPort import SerialPort
from time import sleep
import mkpiano as makeblock
class _BaseBoard:
    MegaPi = "megapi"
    MeOrion = "meorion"
    mCore = "mcore"
    MeAuriga = "meauriga"
    mBuild = "mbuild"
    Halocode = "halocode"
    MegaPiPro = "megapipro"
    CyberPi = "cyberpi"
    Neuron = "neuron"
    _dev = None
    _type = None
    broadcast = None
    def __init__(self,type,device):
        self.__setup(type,device)

    def __setup(self,type,device):
        self._type = type
        self._dev = device
        if type==_BaseBoard.Halocode or type==_BaseBoard.CyberPi:
            self._protocol = HalocodeProtocol()
        elif type==_BaseBoard.mBuild or type==_BaseBoard.Neuron :
            self._protocol = NeuronProtocol()
        else:
            self._protocol = MegaPiProtocol()
        self._dev.setup(self._protocol.on_parse)
        self._protocol.setup(self)
        self._responses = []
        self._subscribe_responses = []
        self._subscribes = {}

    def _autoconnect(self):
        if self._dev is None:
            dev = self.connect()
            if dev is None:
                return False
            sleep(1)
            if not self.broadcast is None:
                self.broadcast()
            return True
    
    def connect(self,device=None):
        if device is None:
            ports = [port[0] for port in SerialPort.list() if port[2] != 'n/a' and port[2].find('1A86:7523')>0 ]
            if len(ports)>0:
                device = SerialPort(ports[0])
                self._dev = device
                self.__setup(self._type,self._dev)
                if self._dev.is_open():
                    return device
                print("\033[1;33m连接失败，设备被占用，请重新拔插指尖钢琴\033[0m")
                # print('$#&*@{"err_code":101,"err_msg":"串口被占用","device":"mkpiano","extra":{}}@*&#$')
            else:
                print("\033[1;33m连接失败，未检测到设备\033[0m")
                print('$#&*@{"err_code":101,"err_msg":"未检测到串口","device":"mkpiano","extra":{}}@*&#$')
        else:
            self._dev = device
            self.__setup(self._type,self._dev)
            return device
    
    def add_thread(self,thread):
        makeblock.add_thread(thread)

    def call(self,pack):
        self._dev.send(pack.to_buffer())

    def request(self,pack):
        if pack.idx==0:
            pack.idx = self._protocol.next_idx
        if pack.subscribe_id in self._subscribes:
            self._subscribes[pack.subscribe_id][pack.cmd] = True
        else:
            self._subscribes[pack.subscribe_id] = {}
            self._subscribes[pack.subscribe_id][pack.cmd] = True
        self._responses.append(pack)
        self._dev.send(pack.to_buffer())

    def repl(self,script):
        buf = bytearray()
        buf.extend(map(ord, script+"\r\n"))
        self._dev.send(buf)

    def subscribe(self,pack):
        pack.subscribe_key = self._protocol.next_subscribe_key
        self._subscribe_responses.append(pack)
        self._dev.send(pack.to_buffer())

    def unsubscribe(self,pack):
        subscribe_pack = self.find_subscribe_response(pack)
        self.remove_subscribe_response(subscribe_pack)
        self._dev.send(pack.to_buffer())

    def on_response(self,pack):
        resp = self.find_response(pack)
        if not resp is None:
            pack.subscribe_id = resp.subscribe_id
            resp.on_response(pack)

    def on_subscribe_response(self,pack):
        resp = self.find_subscribe_response(pack)
        if not resp is None:
            pack.value_name = resp.value_name
            resp.on_response(pack)

    def find_response(self,pack):
        for i in range(len(self._responses)):
            if self._protocol.check_response(pack,self._responses[i]):
                return self._responses[i]
        return None

    def remove_response(self,pack):
        if pack in self._responses:
            self._responses.remove(pack)
    
    def find_subscribe_response(self,pack):
        for i in range(len(self._subscribe_responses)):
            if self._protocol.check_subscribe_response(pack,self._subscribe_responses[i]):
                return self._subscribe_responses[i]
        return None

    def remove_subscribe_response(self,pack):
        if pack in self._subscribe_responses:
            self._subscribe_responses.remove(pack)

    def enable_sending(self):
        self._dev.enable_sending()

    def disable_sending(self):
        self._dev.disable_sending()

    def _find_subscribe(self,subscribe_id,cmd):
        if subscribe_id in self._subscribes:
            if cmd in self._subscribes[subscribe_id]:
                return True
        return False

    @property
    def protocol(self):
        return self._protocol

    @property
    def type(self):
        return self._type
    
from mkpiano.utils import bytes2string
class TestEngine():
    def __init__(self,device,protocol):
        self._dev = device
        self._protocol = protocol
        self._dev.setup(self._protocol.on_parse)
        self._protocol.setup(self)
        self._callback = None

    def on_response(self,resp):
        if not self._callback is None:
            msg = bytes2string(resp)
            try:
                self._callback(eval(msg))
            except(RuntimeError, SyntaxError, TypeError, NameError):
                self._callback(eval("{'err':'json parse err'}"))
        
    def setCallback(self,callback):
        self._callback = callback


# -*- coding: utf-8 -*
import struct
from time import ctime, sleep,time
import threading
import mkpiano
import mkpiano.utils as utils
from mkpiano.protocols.PackData import NeuronPackData

class _BaseModule:
    _pack = None
    _is_received = False
    _last_time = 0
    _callback = None
    def __init__(self,board,idx=1,mode=1,period=0):
        self.setup(board,idx,mode,period)

    def quit(self):
        mkpiano.quit()

    def setup(self,board,idx,mode=1,period=0):
        if board._autoconnect()==False:
            self.quit()
        else:
            print("\033[1;32m连接成功，正在运行程序...\033[0m")
        self._board = board
        self._mode = mode
        self._pack = NeuronPackData()
        self._pack.idx = idx
        self._init_module()
    
    def _init_module(self):
        pass

    def force_update(self):
        self._pack.data = [0x1]
        self.request(self._pack)

    def request(self,pack):
        self._board.remove_response(pack)
        self._board.request(pack)

    def call(self,pack):
        self._board.call(pack)

    def _exist_subscribe(self,subscribe_id,cmd):
        return self._board._find_subscribe(subscribe_id,cmd)

    def subscribe(self,pack=None):
        pass
    _default_data = [0x7f,NeuronPackData.TYPE_CHANGE,0,0,0,0,0]
    def _subscribe(self,idx,service,subservice,on_response,data=None):
        data = data or self._default_data
        subscribe_id = (idx<<24)+(service<<16)+(subservice<<8)
        if self._exist_subscribe(subscribe_id,data[0]):
            return subscribe_id
        pack = NeuronPackData()
        pack.on_response = on_response
        pack.idx = idx
        pack.service = service
        pack.subservice = subservice
        pack.data = data
        pack.cmd = data[0]
        pack.subscribe_id = subscribe_id
        self._board.request(pack)
        return subscribe_id

    def _run_callback(self,value):
        if not self._callback is None:
            self._callback(value)
        return value

    def _on_parse(self, pack):
        self._pack.data = pack.data
        self._is_received = True

    def read(self,idx,service,subservice,data,callback=None):
        if not callback is None:
            self._callback = callback
        self._is_received = False
        pack = NeuronPackData()
        pack.subscribe_id = idx
        pack.idx = idx
        pack.service = service
        pack.subservice = subservice
        pack.data = data
        pack.on_response = self._pack.on_response
        self.request(pack)
        if callback is None:
            timeout = 100
            while not self._is_received:
                timeout-=1
                sleep(0.001)
                if timeout<0:
                    break
            return self._pack.data

class Temperature(_BaseModule):
    def _init_module(self):
        self._temperature = {}
        self._pack.type = NeuronPackData.TYPE_SENSOR
        self._pack.service = 0x63
        self._pack.subservice = 0x1

    def __on_subscribe_response(self, pack):
        if len(pack.data)>6:
            self._temperature[pack.subscribe_id] = utils.bits2float(pack.data[1:6])
        if self._callback is not None:
            self._callback(self._temperature[pack.subscribe_id])

    def on_change(self,callback):
        self._callback = callback

    def get_temperature(self,idx=None):
        subscribe_id = self.__subscribe(idx)
        if subscribe_id in self._temperature:
            return self._temperature[subscribe_id]
        return 0.0

    def __subscribe(self,idx):
        return super()._subscribe(idx or self._pack.idx,self._pack.service,self._pack.subservice,self.__on_subscribe_response)

class Humiture(_BaseModule):
    def _init_module(self):
        self._pack.service = 0x63
        self._pack.subservice = 0x19
        self._status = {"temp":{},"hum":{}}

    def __on_subscribe_response(self, pack):
        if len(pack.data)>3:
            self._status["temp"][pack.subscribe_id] = utils.bits2short(pack.data[1:3])
            self._status["hum"][pack.subscribe_id] = pack.data[3]
        if self._callback is not None:
            self._callback(self._status["temp"][pack.subscribe_id],self._status["hum"][pack.subscribe_id])

    def on_change(self,callback):
        self._callback = callback

    def get_temperature(self,idx=None):
        subscribe_id = self.__subscribe(idx)
        if subscribe_id in self._status["temp"]:
            return self._status["temp"][subscribe_id]
        return 0
    
    def get_humidity(self,idx=None):
        subscribe_id = self.__subscribe(idx)
        if subscribe_id in self._status["hum"]:
            return self._status["hum"][subscribe_id]
        return 0

    def __subscribe(self,idx=None):
        return super()._subscribe(idx or self._pack.idx,self._pack.service,self._pack.subservice,self.__on_subscribe_response)

class Ultrasonic(_BaseModule):
    def _init_module(self):
        self._distance = 0
        self._pack.service = 0x63
        self._pack.subservice = 0x16
        self._pack.on_response = self._on_parse
        self._distance = {}
        
    def _on_parse(self, pack):
        super()._on_parse(pack)
        self._distance[pack.subscribe_id] = super()._run_callback(float('{0:.1f}'.format(utils.bits2float(pack.data[1:6]))))

    def read(self,idx,callback=None):
        res = super().read(idx,self._pack.service,self._pack.subservice,[0x1],callback)
        if idx in self._distance:
            return self._distance[idx]
        return 0.0

    def get_distance(self,idx=0,callback=None):
        idx = idx or self._pack.idx
        return self.read(idx)

class Slider(_BaseModule):
    def _init_module(self):
        self._value = {}
        self._pack.service = 0x64
        self._pack.subservice = 0xd
        self._pack.on_response = self._on_parse
    
    def _on_parse(self, pack):
        super()._on_parse(pack)
        self._value[pack.subscribe_id] = super()._run_callback(pack.data[1])

    def on_change(self,callback):
        self._callback = callback

    def read(self,idx,callback=None):
        res = super().read(idx,self._pack.service,self._pack.subservice,[0x1],callback)
        if idx in self._value:
            return self._value[idx]
        return 0

    def get_value(self,idx=0,callback=None):
        return self.read(idx or self._pack.idx)

class MQ2(_BaseModule):
    def _init_module(self):
        self._value = {}
        self._pack.service = 0x63
        self._pack.subservice = 0x1c
        self.__subscribe()
    
    def __on_subscribe_response(self,pack):
        if len(pack.data)>1:
            self._value[pack.subscribe_id] = pack.data[1]
    
    def get_value(self,idx=None):
        subscribe_id = self.__subscribe(idx)
        if subscribe_id in self._value:
            return self._value[subscribe_id]
        return 0

    def __subscribe(self,idx=None):
        return super()._subscribe(idx or self._pack.idx,self._pack.service,self._pack.subservice,self.__on_subscribe_response)

class Light(_BaseModule):
    def _init_module(self):
        self._value = {}
        self._pack.service = 0x63
        self._pack.subservice = 0x14
    
    def __on_subscribe_response(self,pack):
        if len(pack.data)>1:
            self._value[pack.subscribe_id] = pack.data[1]
    
    def get_value(self,idx=None):
        subscribe_id = self.__subscribe(idx)
        if subscribe_id in self._value:
            return self._value[subscribe_id]
        return 0

    def __subscribe(self,idx=None):
        return super()._subscribe(self._pack.idx,self._pack.service,self._pack.subservice,self.__on_subscribe_response)
        
class SoilMoisture(_BaseModule):
    def _init_module(self):
        self._value = {}
        self._pack.service = 0x63
        self._pack.subservice = 0x15
    
    def __on_subscribe_response(self,pack):
        if len(pack.data)>1:
            self._value[pack.subscribe_id] = pack.data[1]
    
    def get_value(self,idx=None):
        subscribe_id = self.__subscribe(idx)
        if subscribe_id in self._value:
            return self._value[subscribe_id]
        return 0

    def __subscribe(self,idx=None):
        return super()._subscribe(idx or self._pack.idx,self._pack.service,self._pack.subservice,self.__on_subscribe_response)
     
class Sound(_BaseModule):
    def _init_module(self):
        self._value = {}
        self._pack.service = 0x63
        self._pack.subservice = 0x13
    
    def __on_subscribe_response(self,pack):
        if len(pack.data)>1:
            self._value[pack.subscribe_id] = pack.data[1]
    
    @property
    def get_value(self,idx=None):
        subscribe_id = self.__subscribe(idx)
        if subscribe_id in self._value:
            return self._value[subscribe_id]
        return 0

    def __subscribe(self,idx=None):
        return super()._subscribe(idx or self._pack.idx,self._pack.service,self._pack.subservice,self.__on_subscribe_response)

class Touch(_BaseModule):
    def _init_module(self):
        self._value = {}
        self._pack.service = 0x63
        self._pack.subservice = 0xa
    
    def __on_subscribe_response(self,pack):
        if len(pack.data)>1:
            self._value[pack.subscribe_id] = pack.data[1]
    
    def get_value(self,idx=None):
        subscribe_id = self.__subscribe(idx)
        if subscribe_id in self._value:
            return self._value[subscribe_id]
        return 0

    def is_touched(self,index=0,idx=None):
        return ((self.get_value(idx)>>index)&0x1) == 0x1

    def __subscribe(self,idx=None):
        return super()._subscribe(idx or self._pack.idx,self._pack.service,self._pack.subservice,self.__on_subscribe_response)

class Button(_BaseModule):
    def _init_module(self):
        self._pack.service = 0x64
        self._pack.subservice = 0xb
        self._status = {"is_pressed":{},"count":{}}
    
    def __on_subscribe_response(self,pack):
        if len(pack.data)>1:
            if pack.data[0]==1:
                self._status["is_pressed"][pack.subscribe_id] = pack.data[1]==1
            elif pack.data[0]==2:
                self._status["count"][pack.subscribe_id] = utils.bits2long(pack.data[1:6])
    
    def is_pressed(self,idx=0):
        subscribe_id = self.__subscribe(idx)
        if subscribe_id in self._status["is_pressed"]:
            return self._status["is_pressed"][subscribe_id]
        return False
    
    def get_count(self,idx=0):
        subscribe_id = self.__subscribe(idx)
        if subscribe_id in self._status["count"]:
            return self._status["count"][subscribe_id]
        return 0

    def __subscribe(self,idx=None):
        return super()._subscribe(idx or self._pack.idx,self._pack.service,self._pack.subservice,self.__on_subscribe_response)

    def reset(self,idx=None):
        pack = NeuronPackData()
        pack.idx = idx or self._pack.idx
        pack.service = self._pack.service
        pack.subservice = self._pack.subservice
        pack.data = [0x3]
        self.call(pack)

class Joystick(_BaseModule):
    def _init_module(self):
        self._pack.service = 0x64
        self._pack.subservice = 0xc
        self._x = {}
        self._y = {}
        self.__subscribe()
    
    def __on_subscribe_response(self,pack):
        if len(pack.data)>1:
            self._x[pack.subscribe_id] = utils.bits2int8(pack.data[1:3])
            self._y[pack.subscribe_id] = utils.bits2int8(pack.data[3:5])
    
    def get_x(self,idx=None):
        subscribe_id = self.__subscribe(idx)
        if subscribe_id in self._x:
            return self._x[subscribe_id]
        return 0
    
    def get_y(self,idx=None):
        subscribe_id = self.__subscribe(idx)
        if subscribe_id in self._y:
            return self._y[subscribe_id]
        return 0

    def __subscribe(self,idx=None):
        return super()._subscribe(idx or self._pack.idx,self._pack.service,self._pack.subservice,self.__on_subscribe_response)

class LaserRanging(_BaseModule):
    def _init_module(self):
        self._pack.service = 0x63
        self._pack.subservice = 0x12
        self._distance = {}
    
    def __on_subscribe_response(self,pack):
        if len(pack.data)>1:
            self._distance[pack.subscribe_id] = utils.bits2float(pack.data[1:6])
    
    def get_distance(self,idx=None):
        subscribe_id = self.__subscribe(idx)
        if subscribe_id in self._distance:
            return self._distance[subscribe_id]
        return 0

    def __subscribe(self,idx=None):
        return super()._subscribe(idx or self._pack.idx,self._pack.service,self._pack.subservice,self.__on_subscribe_response)

class Flame(_BaseModule):
    def _init_module(self):
        self._pack.service = 0x63
        self._pack.subservice = 0x1b
        self._value = {}
    
    def __on_subscribe_response(self,pack):
        if len(pack.data)>1:
            self._value[pack.subscribe_id] = utils.bits2int8(pack.data[1:3])
    
    def get_value(self,idx=None):
        subscribe_id = self.__subscribe(idx)
        if subscribe_id in self._value:
            return self._value[subscribe_id]
        return 0

    def __subscribe(self,idx=None):
        return super()._subscribe(idx or self._pack.idx,self._pack.service,self._pack.subservice,self.__on_subscribe_response)

class PIRMotion(_BaseModule):
    def _init_module(self):
        self._pack.service = 0x63
        self._pack.subservice = 0x17
        self._status = {"is_moving":{},"count":{}}
    
    def __on_subscribe_response(self,pack):
        if len(pack.data)>1:
            if pack.data[0]==1:
                self._status["is_moving"][pack.subscribe_id] = pack.data[1]==1
            elif pack.data[0]==2:
                self._status["count"][pack.subscribe_id] = utils.bits2long(pack.data[1:6])
    
    def is_moving(self,idx=None):
        subscribe_id = self.__subscribe(idx)
        if subscribe_id in self._status["is_moving"]:
            return self._status["is_moving"][subscribe_id]
        return False
    
    def get_count(self,idx=None):
        subscribe_id = self.__subscribe(idx)
        if subscribe_id in self._status["count"]:
            return self._status["count"][subscribe_id]
        return 0

    def reset(self,idx=None):
        pack = NeuronPackData()
        pack.idx = idx or self._pack.idx
        pack.service = self._pack.service
        pack.subservice = self._pack.subservice
        pack.data = [0x3]
        self.call(pack)

    def __subscribe(self,idx=None):
        return super()._subscribe(idx or self._pack.idx,self._pack.service,self._pack.subservice,self.__on_subscribe_response)

class Magnetic(_BaseModule):
    def _init_module(self):
        self._pack.service = 0x63
        self._pack.subservice = 0x1e
        self._status = {"is_magnetic":{},"count":{}}
    
    def __on_subscribe_response(self,pack):
        if len(pack.data)>1:
            if pack.data[0]==1:
                self._status["is_magnetic"][pack.subscribe_id] = pack.data[1]==1
            elif pack.data[0]==2:
                self._status["count"][pack.subscribe_id] = utils.bits2long(pack.data[1:6])
    
    def is_magnetic(self,idx=None):
        subscribe_id = self.__subscribe(idx)
        if subscribe_id in self._status['is_magnetic']:
            return self._status["is_magnetic"][subscribe_id]
        return False
    
    def get_count(self,idx=None):
        subscribe_id = self.__subscribe(idx)
        if subscribe_id in self._status['count']:
            return self._status["count"][subscribe_id]
        return 0

    def reset(self,idx=None):
        pack = NeuronPackData()
        pack.idx = idx or self._pack.idx
        pack.service = self._pack.service
        pack.subservice = self._pack.subservice
        pack.data = [0x3]
        self.call(pack)

    def __subscribe(self,idx=None):
        return super()._subscribe(idx or self._pack.idx,self._pack.service,self._pack.subservice,self.__on_subscribe_response)

class Angle(_BaseModule):
    def _init_module(self):
        self._pack.service = 0x63
        self._pack.subservice = 0xe
        self._status = {"direction":{},"speed":{},"angle":{}}
    
    def __on_subscribe_response(self,pack):
        if len(pack.data)>1:
            if pack.data[0]==1:
                self._status["angle"][pack.subscribe_id] = utils.bits2long(pack.data[1:6])
            elif pack.data[0]==2:
                self._status["direction"][pack.subscribe_id] = pack.data[1]
            elif pack.data[0]==4:
                self._status["speed"][pack.subscribe_id] = utils.bits2long(pack.data[1:6])
    
    def get_angle(self,idx=None):
        subscribe_id = self.__subscribe(idx)
        if subscribe_id in self._status['angle']:
            return self._status["angle"][subscribe_id]
        return 0
    
    def get_direction(self,idx=None):
        subscribe_id = self.__subscribe(idx)
        if subscribe_id in self._status['direction']:
            return self._status["direction"][subscribe_id]
        return 0

    def get_speed(self):
        subscribe_id = self.__subscribe(idx)
        if subscribe_id in self._status['speed']:
            return self._status["speed"][subscribe_id]
        return 0

    def reset(self,idx=None):
        pack = NeuronPackData()
        pack.idx = idx or self._pack.idx
        pack.service = self._pack.service
        pack.subservice = self._pack.subservice
        pack.data = [0x3]
        self.call(pack)

    def __subscribe(self,idx=None):
        return super()._subscribe(idx or self._pack.idx,self._pack.service,self._pack.subservice,self.__on_subscribe_response)

class Motion(_BaseModule):
    def _init_module(self):
        self._pack.service = 0x63
        self._pack.subservice = 0x1a
        self._status = {"intensity":{},"acceleration_x":{},"acceleration_y":{},"acceleration_z":{},"velocity_x":{},"velocity_y":{},"velocity_z":{},"pitch":{},"roll":{}}
    
    def __on_subscribe_response(self,pack):
        if len(pack.data)>1:
            if pack.data[0]==1:
                self._status["intensity"][pack.subscribe_id] = pack.data[1]
            elif pack.data[0]==2:
                self._status["acceleration_x"][pack.subscribe_id] = utils.bits2float(pack.data[1:6])
            elif pack.data[0]==3:
                self._status["acceleration_y"][pack.subscribe_id] = utils.bits2float(pack.data[1:6])
            elif pack.data[0]==4:
                self._status["acceleration_z"][pack.subscribe_id] = utils.bits2float(pack.data[1:6])
            elif pack.data[0]==5:
                self._status["velocity_x"][pack.subscribe_id] = utils.bits2float(pack.data[1:6])
            elif pack.data[0]==6:
                self._status["velocity_y"][pack.subscribe_id] = utils.bits2float(pack.data[1:6])
            elif pack.data[0]==7:
                self._status["velocity_z"][pack.subscribe_id] = utils.bits2float(pack.data[1:6])
            elif pack.data[0]==8:
                self._status["pitch"][pack.subscribe_id] = utils.bits2short(pack.data[1:3])
            elif pack.data[0]==9:
                self._status["roll"][pack.subscribe_id] = utils.bits2short(pack.data[1:3])
    
    def get_intensity(self,idx=None):
        subscribe_id = self.__subscribe(idx)
        if subscribe_id in self._status['intensity']:
            return self._status["intensity"][subscribe_id]
        return 0
    
    def get_acceleration_x(self,idx=None):
        subscribe_id = self.__subscribe(idx)
        if subscribe_id in self._status['intensity']:
            return self._status["acceleration_x"][subscribe_id]
        return 0

    def get_acceleration_y(self,idx=None):
        subscribe_id = self.__subscribe(idx)
        if subscribe_id in self._status['intensity']:
            return self._status["acceleration_y"][subscribe_id]
        return 0

    def get_acceleration_z(self,idx=None):
        subscribe_id = self.__subscribe(idx)
        if subscribe_id in self._status['intensity']:
            return self._status["acceleration_z"][subscribe_id]
        return 0

    def get_velocity_x(self,idx=None):
        subscribe_id = self.__subscribe(idx)
        if subscribe_id in self._status['intensity']:
            return self._status["velocity_x"][subscribe_id]
        return 0

    def get_velocity_y(self,idx=None):
        subscribe_id = self.__subscribe(idx)
        if subscribe_id in self._status['intensity']:
            return self._status["velocity_y"][subscribe_id]
        return 0

    def get_velocity_z(self,idx=None):
        subscribe_id = self.__subscribe(idx)
        if subscribe_id in self._status['intensity']:
            return self._status["velocity_z"][subscribe_id]
        return 0

    def get_pitch(self,idx=None):
        subscribe_id = self.__subscribe(idx)
        if subscribe_id in self._status['intensity']:
            return self._status["pitch"][subscribe_id]
        return 0

    def get_roll(self,idx=None):
        subscribe_id = self.__subscribe(idx)
        if subscribe_id in self._status['intensity']:
            return self._status["roll"][subscribe_id]
        return 0

    def reset(self,idx=None):
        pack = NeuronPackData()
        pack.idx = idx or self._pack.idx
        pack.service = self._pack.service
        pack.subservice = self._pack.subservice
        pack.data = [0x4]
        self.call(pack)
        pack.data = [0x5]
        self.call(pack)
        pack.data = [0x6]
        self.call(pack)

    def __subscribe(self,idx=None):
        for i in range(9):
            super()._subscribe(self._pack.idx,self._pack.service,self.__on_subscribe_response,[0x1,NeuronPackData.TYPE_CHANGE,i+1,0,0,0,0,0])


class Color(_BaseModule):
    def _init_module(self):
        self._pack.service = 0x63
        self._pack.subservice = 0x11
        self._pack.on_response = self._on_parse
        self._color = {}
        self._intensity = {}
        self._reflect = {}
        
    def _on_parse(self, pack):
        super()._on_parse(pack)
        if len(pack.data)>1:
            if pack.data[0]==0x8:
                self._color = super()._run_callback({'red1':utils.bits2short(pack.data[1:3]),'green1':utils.bits2short(pack.data[3:5]),'blue1':utils.bits2short(pack.data[5:7]),'red2':utils.bits2short(pack.data[7:9]),'green2':utils.bits2short(pack.data[9:11]),'blue2':utils.bits2short(pack.data[11:13])})
            elif pack.data[0]==0x9:
                self._intensity = super()._run_callback([utils.bits2short(pack.data[1:3]),utils.bits2short(pack.data[3:5])])
            elif pack.data[0]==0xa:
                self._reflect = super()._run_callback([utils.bits2short(pack.data[1:3]),utils.bits2short(pack.data[3:5])])

    def get_color(self,idx=0,callback=None):
        idx = idx or self._pack.idx
        super().read(idx,self._pack.service,self._pack.subservice,[0x8],callback)
        return self._color

    def get_intensity(self,idx=0,callback=None):
        idx = idx or self._pack.idx
        super().read(idx,self._pack.service,self._pack.subservice,[0x9],callback)
        return self._intensity

    def get_reflect(self,idx=0,callback=None):
        idx = idx or self._pack.idx
        super().read(idx,self._pack.service,self._pack.subservice,[0xa],callback)
        return self._reflect

class GPIO(_BaseModule):
    def _init_module(self):
        self._pack.service = 0x68
        self._pack.subservice = 0x1
        self._callback = None
        self._value = {}

    def __on_parse(self,pack):
        super()._on_parse(pack)
        if pack.data[0]==3:
            self._value[pack.subscribe_id] = super()._run_callback(utils.bits2short(pack.data[1:3]))
        elif pack.data[0]==2:
            self._value[pack.subscribe_id] = super()._run_callback(pack.data[1])

    def digital_write(self,pin,level,idx=None):
        pack = NeuronPackData()
        pack.idx = idx or self._pack.idx
        pack.service = self._pack.service
        pack.subservice = self._pack.subservice
        pack.data = [0x1,pin,level]
        self.call(pack)

    def pwm_write(self,pin,pwm,freq=1000,idx=None):
        pack = NeuronPackData()
        pack.idx = idx or self._pack.idx
        pack.service = self._pack.service
        pack.subservice = self._pack.subservice
        pack.data = [0x4,pin]
        pack.data.extend(utils.float2bits(freq))
        pack.data.extend(utils.float2bits(pwm))
        self.call(pack)

    def digital_read(self,pin,idx=None,callback=None):
        idx = idx or self._pack.idx
        res = super().read(idx,self._pack.service,self._pack.subservice,[0x2,pin],callback)
        if idx in self._value:
            return self._value[idx]
        return 0

    def analog_read(self,pin,idx=None,callback=None):
        idx = idx or self._pack.idx
        res = super().read(idx,self._pack.service,self._pack.subservice,[0x3,pin],callback)
        if idx in self._value:
            return self._value[idx]
        return 0

    def enable(self,status=0,idx=None):
        pack = NeuronPackData()
        pack.idx = idx or self._pack.idx
        pack.service = self._pack.service
        pack.subservice = self._pack.subservice
        pack.data = [0x5,status]
        self.call(pack)

class PowerManager(_BaseModule):
    def _init_module(self):
        self._pack.service = 0x64
        self._pack.subservice = 0x9
        self._pack.on_response = self.__on_subscribe_response
        self._status = {}
        self._wireless = {}
        self._power = {}

    def __on_subscribe_response(self,pack):
        if len(pack.data)>1:
            if pack.data[0]==0x1:
                self._wireless[pack.subscribe_id] = pack.data[1]
                self._power[pack.subscribe_id] = pack.data[2]
            elif pack.data[0]==0x2:
                super()._on_parse(pack)
                self._status = super()._run_callback({'voltage':utils.bits2float(pack.data[1:6]),'current':utils.bits2float(pack.data[6:11])})

    def get_status(self,idx=None,callback=None):
        idx = idx or self._pack.idx
        res = super().read(idx,self._pack.service,self._pack.subservice,[0x2],callback)
        return self._status

    def is_wireless(self,idx=None):
        subscribe_id = self.__subscribe(idx)
        if subscribe_id in self._status['wireless']:
            return self._status["wireless"][subscribe_id]
        return True

    def is_power(self,idx=None):
        subscribe_id = self.__subscribe(idx)
        if subscribe_id in self._status['power']:
            return self._status["power"][subscribe_id]
        return True

    def __subscribe(self,idx=None):
        return super()._subscribe(idx or self._pack.idx,self._pack.service,self._pack.subservice,self.__on_subscribe_response,[0x7f,NeuronPackData.TYPE_CHANGE,0,0,0,0,0])

class Infrarer(_BaseModule):
    def _init_module(self):
        self._pack.service = 0x68
        self._pack.subservice = 0x2
    
    def send_message(self,msg="",idx=None):
        pack = NeuronPackData()
        pack.idx = idx or self._pack.idx
        pack.service = self._pack.service
        pack.subservice = self._pack.subservice
        pack.data = [0x2,len(msg)]
        pack.data.extend(utils.string2bytes(msg))
        self.call(pack)

    def record(self,index=0,time=3000,idx=None):
        if index>1:
            index=1
        if index<0:
            index=0
        pack = NeuronPackData()
        pack.idx = idx or self._pack.idx
        pack.service = self._pack.service
        pack.subservice = self._pack.subservice
        pack.data = [0x3]
        pack.data.extend(utils.long2bits(time))
        pack.data.append(index)
        self.call(pack)

    def send_record(self,index=0,idx=None):
        if index>1:
            index=1
        if index<0:
            index=0
        pack = NeuronPackData()
        pack.idx = idx or self._pack.idx
        pack.service = self._pack.service
        pack.subservice = self._pack.subservice
        pack.data = [0x4,index]
        self.call(pack)

class RGBLed(_BaseModule):
    def _init_module(self):
        self._pack.service = 0x65
        self._pack.subservice = 0x2
    
    def set_color(self,red,green,blue,idx=None):
        self._pack.idx = idx or self._pack.idx
        self._pack.data = [0x1]
        self._pack.data.extend(utils.ushort2bits(red))
        self._pack.data.extend(utils.ushort2bits(green))
        self._pack.data.extend(utils.ushort2bits(blue))
        self.call(self._pack)
    
class LedStrip(_BaseModule):
    def _init_module(self):
        self._pack.service = 0x65
        self._pack.subservice = 0x3
    
    def set_color(self,index,red,green,blue,idx=None):
        self._pack.idx = idx or self._pack.idx
        self._pack.data = [0x1,index]
        self._pack.data.extend(utils.ushort2bits(red))
        self._pack.data.extend(utils.ushort2bits(green))
        self._pack.data.extend(utils.ushort2bits(blue))
        self.call(self._pack)

class LedMatrix(_BaseModule):
    def _init_module(self):
        self._pack.service = 0x65
        self._pack.subservice = 0x9
    
    def set_pixel(self,index,red,green,blue,idx=None):
        self._pack.idx = idx or self._pack.idx
        self._pack.data = [0x2,index]
        self._pack.data.extend(utils.ushort2bits(red))
        self._pack.data.extend(utils.ushort2bits(green))
        self._pack.data.extend(utils.ushort2bits(blue))
        self.call(self._pack)
    
    def set_pixels(self,bits,red,green,blue,idx=None):
        self._pack.idx = idx or self._pack.idx
        self._pack.data = [0x1]
        for i in range(2):
            self._pack.data.extend(utils.long2bits(bits[i]))
        self._pack.data.extend(utils.ushort2bits(red))
        self._pack.data.extend(utils.ushort2bits(green))
        self._pack.data.extend(utils.ushort2bits(blue))
        self.call(self._pack)

    def set_string(self,msg,idx=None):
        self._pack.idx = idx or self._pack.idx
        l = len(msg)
        self._pack.data = [0x7]
        self._pack.data.extend(utils.ushort2bits(l))
        for i in range(l):
            self._pack.data.append(ord(msg[i]))
        self.call(self._pack)

class Servo(_BaseModule):
    BOTH_SERVOS = 1
    LEFT_SERVO = 2
    RIGHT_SERVO = 3
    def _init_module(self):
        self._pack.service = 0x62
        self._pack.subservice = 0xa
    
    def set_angle(self,angle,idx=None):
        self._pack.idx = idx or self._pack.idx
        angle = int(angle)
        self._pack.data = [0x1]
        self._pack.data.extend(utils.ushort2bits(angle))
        self.call(self._pack)
    
    def release(self,idx=None):
        self._pack.idx = idx or self._pack.idx
        self._pack.data = [0x6]
        self.call(self._pack)

class DCMotor(_BaseModule):
    def _init_module(self):
        self._pack.service = 0x62
        self._pack.subservice = 0x9
    
    def set_pwm(self,pwm,idx=None):
        self._pack.idx = idx or self._pack.idx
        self._pack.data = [0x1]
        self._pack.data.extend(utils.int2bits(pwm))
        self.call(self._pack)


class EncoderMotor(_BaseModule):
    def _init_module(self):
        self._pack.service = 0x62
        self._pack.subservice = 0x6
        self._stall = {}

    def set_speed(self,speed,idx=None):
        if speed>320:
            speed = 320
        if speed<-320:
            speed = -320
        pack = NeuronPackData()
        pack.idx = idx or self._pack.idx
        pack.service = self._pack.service
        pack.subservice = self._pack.subservice
        pack.data = [0x4]
        pack.data.extend(utils.ushort2bits(speed))
        self.call(pack)

    def set_pwm(self,pwm,idx=None):
        if pwm>255:
            pwm = 255
        if pwm<-255:
            pwm = -255
        pack = NeuronPackData()
        pack.idx = idx or self._pack.idx
        pack.service = self._pack.service
        pack.subservice = self._pack.subservice
        pack.data = [0x11]
        pack.data.extend(utils.ushort2bits(pwm))
        self.call(pack)

    def move_to(self,position,speed,idx=None):
        if speed>320:
            speed = 320
        if speed<0:
            speed = 0
        pack = NeuronPackData()
        pack.idx = idx or self._pack.idx
        pack.service = self._pack.service
        pack.subservice = self._pack.subservice
        pack.data = [0x2]
        pack.data.extend(utils.long2bits(position))
        pack.data.extend(utils.ushort2bits(speed))
        self.call(pack)

    def move(self,position,speed,idx=None):
        if speed>320:
            speed = 320
        if speed<0:
            speed = 0
        pack = NeuronPackData()
        pack.idx = idx or self._pack.idx
        pack.service = self._pack.service
        pack.subservice = self._pack.subservice
        pack.data = [0x3]
        pack.data.extend(utils.long2bits(position))
        pack.data.extend(utils.ushort2bits(speed))
        self.call(pack)

    def set_lock(self,lock,idx=None):
        if lock>1:
            lock = 1
        if lock<0:
            lock = 0
        pack = NeuronPackData()
        pack.idx = idx or self._pack.idx
        pack.service = self._pack.service
        pack.subservice = self._pack.subservice
        pack.data = [0x14,lock]
        self.call(pack)

    def set_zero(self,idx=None):
        pack = NeuronPackData()
        pack.idx = idx or self._pack.idx
        pack.service = self._pack.service
        pack.subservice = self._pack.subservice
        pack.data = [0x12]
        self.call(pack)

    def stop(self,idx = None):
        pack = NeuronPackData()
        pack.idx = idx or self._pack.idx
        pack.service = self._pack.service
        pack.subservice = self._pack.subservice
        pack.data = [0x5]
        self.call(pack)

    def __on_subscribe_response(self,pack):
        if pack.data[0]==0x27:
            self._status["stall"] = pack.data[1]==1
    
    def get_stall(self,idx=None):
        subscribe_id = self.__subscribe(idx)
        if subscribe_id in self._stall:
            return self._stall[subscribe_id]
        return False

    def __subscribe(self,idx=None):
        super()._subscribe(idx or self._pack.idx,self._pack.service,self._pack.subservice,[0x7d,NeuronPackData.TYPE_CHANGE,0,0,0,0,0])

class ExtDCMotor(_BaseModule):
    def _init_module(self):
        self._pack.service = 0x62
        self._pack.subservice = 0x7

    def reset(self,port,idx=None):
        pack = NeuronPackData()
        pack.idx = idx or self._pack.idx
        pack.service = self._pack.service
        pack.subservice = self._pack.subservice
        pack.data = [0x1,port]
        self.call(pack)

    def set_pwm(self,port,pwm,idx=None):
        pack = NeuronPackData()
        pack.idx = idx or self._pack.idx
        pack.service = self._pack.service
        pack.subservice = self._pack.subservice
        pack.data = [0x2,port]
        pack.data.extend(utils.ushort2bits(pwm))
        self.call(pack)

    def stop(self,port,idx=None):
        pack = NeuronPackData()
        pack.idx = idx or self._pack.idx
        pack.service = self._pack.service
        pack.subservice = self._pack.subservice
        pack.data = [0x3,port]
        self.call(pack)

class SmartServo(_BaseModule):
    def _init_module(self):
        self._pack.service = 0x60
        self._status = {"speed":{},"position":{},"temp":{},"current":{},"voltage":{}}

    def __on_parse(self,pack):
        super()._on_parse(pack)
        if pack.subservice == 0x41:
            super()._run_callback(pack.data[0])

    def __on_subscribe_response(self,pack):
        if pack.subservice==0x36:
            self._status["position"][pack.subscribe_id] = utils.bits2long(pack.data[0:5])
        elif pack.subservice==0x23:
            self._status["speed"][pack.subscribe_id] = int(utils.bits2float(pack.data[0:5])*10)/10
        elif pack.subservice==0x25:
            self._status["temp"][pack.subscribe_id] = int(utils.bits2float(pack.data[0:5])*100)/100
        elif pack.subservice==0x26:
            self._status["current"][pack.subscribe_id] = int(utils.bits2float(pack.data[0:5])*100)/100
        elif pack.subservice==0x27:
            self._status["voltage"][pack.subscribe_id] = int(utils.bits2float(pack.data[0:5])*100)/100

    def __subscribe(self,subservice,idx=None):
        idx = idx or self._pack.idx
        return super()._subscribe(idx,self._pack.service,subservice,self.__on_subscribe_response,[NeuronPackData.TYPE_CHANGE])

    def get_position(self,idx=None):
        subscribe_id = self.__subscribe(0x36,idx)
        if subscribe_id in self._status["position"]:
            return self._status["position"][subscribe_id]
        return 0

    def get_speed(self,idx=None):
        subscribe_id = self.__subscribe(0x23,idx)
        if subscribe_id in self._status["speed"]:
            return self._status["speed"][subscribe_id]
        return 0

    def get_voltage(self,idx=None):
        subscribe_id = self.__subscribe(0x27,idx)
        if subscribe_id in self._status["voltage"]:
            return self._status["voltage"][subscribe_id]
        return 0

    def get_current(self,idx=None):
        subscribe_id = self.__subscribe(0x26,idx)
        if subscribe_id in self._status["current"]:
            return self._status["current"][subscribe_id]
        return 0

    def get_temperature(self,idx=None):
        subscribe_id = self.__subscribe(0x25,idx)
        if subscribe_id in self._status["temp"]:
            return self._status["temp"][subscribe_id]
        return 0

    def set_lock(self,lock):
        pack = NeuronPackData()
        pack.idx = idx or self._pack.idx
        pack.service = self._pack.service
        pack.subservice = 0x16
        pack.data = [lock]
        self.call(pack)

    def set_led(self,red,green,blue):
        pack = NeuronPackData()
        pack.idx = idx or self._pack.idx
        pack.service = self._pack.service
        pack.subservice = 0x17
        pack.data = []
        pack.data.extend(utils.ushort2bits(red))
        pack.data.extend(utils.ushort2bits(green))
        pack.data.extend(utils.ushort2bits(blue))
        self.call(pack)

    def set_zero(self):
        pack = NeuronPackData()
        pack.service = self._pack.service
        pack.subservice = 0x30
        self.call(pack)

    def move_to(self,position,speed,idx=None):
        pack = NeuronPackData()
        pack.idx = idx or self._pack.idx
        pack.service = self._pack.service
        pack.subservice = 0x33
        pack.data = []
        pack.data.extend(utils.long2bits(position))
        pack.data.extend(utils.ushort2bits(speed))
        self.call(pack)

    def move(self,position,speed,idx=None):
        pack = NeuronPackData()
        pack.idx = idx or self._pack.idx
        pack.service = self._pack.service
        pack.subservice = 0x34
        pack.data = []
        pack.data.extend(utils.long2bits(position))
        pack.data.extend(utils.ushort2bits(speed))
        self.call(pack)
    
    def set_rotation(self,rotation,during,idx=None):
        pack = NeuronPackData()
        pack.idx = idx or self._pack.idx
        pack.service = self._pack.service
        pack.subservice = 0x2a
        pack.data = []
        pack.data.extend(utils.float2bits(rotation))
        pack.data.extend(utils.ushort2bits(during))
        self.call(pack)

    def set_pwm(self,pwm,idx=None):
        pack = NeuronPackData()
        pack.idx = idx or self._pack.idx
        pack.service = self._pack.service
        pack.subservice = 0x35
        pack.data = []
        pack.data.extend(utils.short2bits(pwm))
        self.call(pack)

    def set_absolute_angle_with_force(self,angle,speed,force=100,idx=None):
        pack = NeuronPackData()
        pack.idx = idx or self._pack.idx
        pack.service = self._pack.service
        pack.subservice = 0x38
        pack.data = []
        pack.data.extend(utils.long2bits(angle))
        pack.data.extend(utils.ushort2bits(speed))
        pack.data.extend(utils.ushort2bits(force))
        self.call(pack)

    def set_relative_angle_with_force(self,angle,speed,force=100,idx=None):
        pack = NeuronPackData()
        pack.idx = idx or self._pack.idx
        pack.service = self._pack.service
        pack.subservice = 0x39
        pack.data = []
        pack.data.extend(utils.long2bits(angle))
        pack.data.extend(utils.ushort2bits(speed))
        pack.data.extend(utils.ushort2bits(force))
        self.call(pack)

    def set_speed(self,speed,idx=None):
        pack = NeuronPackData()
        pack.idx = idx or self._pack.idx
        pack.service = self._pack.service
        pack.subservice = 0x3a
        pack.data = []
        pack.data.extend(utils.int2bits(speed))
        self.call(pack)

    def home(self,direction,speed,idx=None):
        pack = NeuronPackData()
        pack.idx = idx or self._pack.idx
        pack.service = self._pack.service
        pack.subservice = 0x43
        pack.data = [direction]
        pack.data.extend(utils.ushort2bits(speed))
        self.call(pack)

    def get_error(self,idx=None,callback=None):
        self._pack.on_response = self.__on_parse
        res = super().read(idx or self._pack.idx,self._pack.service,0x41,[],callback)
        return res[0]

    def clear_error(self,idx=None):
        pack = NeuronPackData()
        pack.idx = idx or self._pack.idx
        pack.service = self._pack.service
        pack.subservice = 0x42
        self.call(pack)

class Speaker(_BaseModule):
    def _init_module(self):
        self._pack.service = 0x66
        self._pack.subservice = 0x4

    def play_tone(self,hz,idx=None):
        self._pack.idx = idx or self._pack.idx
        self._pack.data = [0x2]
        self._pack.data.extend(utils.ushort2bits(hz))
        self.call(self._pack)
    
class FingertipPiano(_BaseModule):
    '''
        :description: Fingertip Piano
        :example:
        .. code-block:: python
            :linenos:

            piano = FingertipPiano(1)

    '''
    def _init_module(self):
        self._isExiting = False
        self._pack.service = 0x64
        self._pack.subservice = 0xf
        self._hasCount = False
        self._piano = {'touch_strength':{},'key':{},'touch':{},'joystick':{},'distance':{},'key_count':{},'last_touched':{},'last_pressed':{},'gesture':{}}

    def __on_parse(self, pack):
        pass

    def __on_subscribe_response(self,pack):
        if pack.data[0]==0x2:
            self._piano['key'][pack.subscribe_id] = pack.data[1:3]
        elif pack.data[0]==0x3:
            self._piano['touch'][pack.subscribe_id] = pack.data[1:5]
        elif pack.data[0]==0x6:
            self._piano['gesture'][pack.subscribe_id] = pack.data[1]
        elif pack.data[0]==0x5:
            self._piano['joystick'][pack.subscribe_id] = [utils.bits2int8(pack.data[1:3]),utils.bits2int8(pack.data[3:5])]
        elif pack.data[0]==0x7:
            self._piano['distance'][pack.subscribe_id] = pack.data[1]
        elif pack.data[0]==0x8:
            self._piano['touch_strength'][pack.subscribe_id] = [utils.bits2short(pack.data[1:3]),utils.bits2short(pack.data[3:5]),utils.bits2short(pack.data[5:7]),utils.bits2short(pack.data[7:9])]
        elif pack.data[0]==0x9:
            self._piano['key_count'][pack.subscribe_id] = [utils.bits2long(pack.data[1:6]),utils.bits2long(pack.data[6:11])]

    def __subscribe(self,type,idx=None,mode=NeuronPackData.TYPE_CHANGE):
        idx = idx or self._pack.idx
        return super()._subscribe(idx,self._pack.service,self._pack.subservice,self.__on_subscribe_response,[type,mode]+utils.long2bits(100))

    def count_pressed(self,index=1,idx=None):
        return self.get_count_pressed(index,idx)

    def get_count_pressed(self,index=1,idx=None):
        subscribe_id = self.__subscribe(0x78,idx)
        if self._hasCount == False:
            self._hasCount = True
            data = super().read(idx or self._pack.idx,self._pack.service,self._pack.subservice,[0x9])
        if subscribe_id in self._piano['key_count']:
            if index==1 or (type(index)==str and index.lower()=='a'):
                return self._piano['key_count'][subscribe_id][0]
            if index==2 or (type(index)==str and index.lower()=='b'):
                return self._piano['key_count'][subscribe_id][1]
        return 0

    def is_pressed(self,index=1,idx=None):
        subscribe_id = self.__subscribe(0x7f,idx)
        if subscribe_id in self._piano['key']:
            if index==1 or (type(index)==str and index.lower()=='a'):
                return self._piano['key'][subscribe_id][0] == 1
            if index==2 or (type(index)==str and index.lower()=='b'):
                return self._piano['key'][subscribe_id][1] == 1
        return False

    def get_pressed(self,index=1,idx=None):
        return self.is_pressed(index,idx)

    @property
    def is_obstacle_ahead(self):
        return self.distance<95

    def get_obstacle_ahead(self):
        return self.distance<95

    @property
    def obstacle_moving(self):
        return self.gesture

    def get_obstacle_moving(self):
        return self.gesture

    def last_pressed(self,idx=0):
        if idx==1:
            idx = 'A'
        elif idx==2:
            idx = 'B'
        if idx==0:
            if (self._piano['keyA']+self._piano['keyB'])==0 and self._piano['last_pressed']!=idx:
                self._piano['last_pressed']=idx
                return False
        elif self._piano['key'+idx] and self._piano['last_pressed']!=idx:
            self._piano['last_pressed']=idx
            return False
        return True

    def touch_value(self,index=1,idx=None):
        subscribe_id = self.__subscribe(0x79,idx,NeuronPackData.TYPE_PERIOD)
        if subscribe_id in self._piano['touch_strength']:
            if index==1 or (type(index)==str and index.lower()=='a'):
                return min(100,int(self._piano['touch_strength'][subscribe_id][0]*160/255))
            elif index==2 or (type(index)==str and index.lower()=='b'):
                return min(100,int(self._piano['touch_strength'][subscribe_id][1]*160/255))
            elif index==3 or (type(index)==str and index.lower()=='c'):
                return min(100,int(self._piano['touch_strength'][subscribe_id][2]*160/255))
            elif index==4 or (type(index)==str and index.lower()=='d'):
                return min(100,int(self._piano['touch_strength'][subscribe_id][3]*160/255))
        return 0


    def get_touch_value(self,index=1,idx=None):
        return self.touch_value(index,idx)
        
    def is_touched(self,index=1,idx=None):
        subscribe_id = self.__subscribe(0x7e,idx)
        if subscribe_id in self._piano['touch']:
            if index==1 or (type(index)==str and index.lower()=='a'):
                return self._piano['touch'][subscribe_id][0]==1
            elif index==2 or (type(index)==str and index.lower()=='b'):
                return self._piano['touch'][subscribe_id][1]==1
            elif index==3 or (type(index)==str and index.lower()=='c'):
                return self._piano['touch'][subscribe_id][2]==1
            elif index==4 or (type(index)==str and index.lower()=='d'):
                return self._piano['touch'][subscribe_id][3]==1
        return False

    def get_touched(self,index=1,idx=None):
        return self.is_touched(index,idx)
        
    def last_touched(self,idx=0):
        if idx==0:
            if self._piano['touch4']+self._piano['touch1']+self._piano['touch2']+self._piano['touch3']==0 and self._piano['last_touched']!=idx:
                self._piano['last_touched']=idx
                return 0
        elif self._piano['touch'+str(idx)] and self._piano['last_touched']!=idx:
            self._piano['last_touched']=idx
            return 0
        return 1

    @property
    def gesture(self):
        return self.get_gesture()

    def get_gesture(self,idx=None):
        subscribe_id = self.__subscribe(0x7b,idx)
        if subscribe_id in self._piano['gesture']:
            return self._piano['gesture'][subscribe_id]
        return 0

    @property
    def distance(self):
        return self.get_distance()

    def get_distance(self,idx=None):
        subscribe_id = self.__subscribe(0x7a,idx,NeuronPackData.TYPE_PERIOD)
        if subscribe_id in self._piano['distance']:
            return self._piano['distance'][subscribe_id]
        return 100

    @property
    def ir_intensity(self):
        return 100 - self.distance

    def get_ir_intensity(self,idx=None):
        return 100 - self.get_distance(idx)

    @property
    def joystick_x(self):
        return self.get_joystick_x()

    def get_joystick_x(self,idx=None):
        subscribe_id = self.__subscribe(0x7c,idx,NeuronPackData.TYPE_PERIOD)
        if subscribe_id in self._piano['joystick']:
            return self._piano['joystick'][subscribe_id][0]
        return 0

    @property
    def joystick_y(self):
        return self.get_joystick_y()

    def get_joystick_y(self,idx=None):
        subscribe_id = self.__subscribe(0x7c,idx,NeuronPackData.TYPE_PERIOD)
        if subscribe_id in self._piano['joystick']:
            return self._piano['joystick'][subscribe_id][1]
        return 0

    @property
    def joystick_direction(self):
        if self.is_joystick_up():
            return self.DIRECTION_UP
        elif self.is_joystick_down():
            return self.DIRECTION_DOWN
        elif self.is_joystick_left():
            return self.DIRECTION_LEFT
        elif self.is_joystick_right():
            return self.DIRECTION_RIGHT
        return self.DIRECTION_HOME

    def get_joystick_direction(self):
        return self.joystick_direction
        
    def is_joystick_up(self):
        return self.get_joystick_y()>30

    def is_joystick_down(self):
        return self.get_joystick_y()<-30

    def is_joystick_left(self):
        return self.get_joystick_x()<-30

    def is_joystick_right(self):
        return self.get_joystick_x()>30
    
    @property
    def DIRECTION_HOME(self):
        return 0

    @property
    def DIRECTION_UP(self):
        return 1

    @property
    def DIRECTION_DOWN(self):
        return 2

    @property
    def DIRECTION_LEFT(self):
        return 3

    @property
    def DIRECTION_RIGHT(self):
        return 4

    @property  
    def GESTURE_NONE(self):
        return 0

    @property  
    def GESTURE_WAVING(self):
        return 1
    
    @property  
    def GESTURE_MOVING_DOWN(self):
        return 2

    @property  
    def GESTURE_MOVING_UP(self):
        return 3

    @property
    def APPROACHING(self):
        return self.GESTURE_MOVING_DOWN

    @property
    def AWAY(self):
        return self.GESTURE_MOVING_UP

    def set_led(self,red=0,green=0,blue=0,idx=None):
        if type(red)==str or red<0:
            red = 0
        if type(green)==str or green<0:
            green = 0
        if type(blue)==str or blue<0:
            blue = 0
        pack = NeuronPackData()
        pack.idx = idx or self._pack.idx
        pack.service = self._pack.service
        pack.subservice = self._pack.subservice
        pack.data = [0x1]
        pack.data.extend(utils.ushort2bits(red))
        pack.data.extend(utils.ushort2bits(green))
        pack.data.extend(utils.ushort2bits(blue))
        self.call(pack)

    def led_on(self,red=0,green=0,blue=0,idx=None):
        self.set_led(red,green,blue,idx)

    def led_off(self,idx=None):
        self.set_led(0,0,0,idx)

    def reset_button(self,index=2,idx=None):
        self.reset_pressed(index,idx)

    def reset_pressed(self,index=2,idx=None):
        if type(index)==str:
            if index.lower()=='a':
                index = 0
            elif index.lower()=='b':
                index = 1
        pack = NeuronPackData()
        pack.idx = idx or self._pack.idx
        pack.service = self._pack.service
        pack.subservice = self._pack.subservice
        pack.data = [0x11,index]
        self.call(pack)
    
    def calibrate(self,idx=None):
        pack = NeuronPackData()
        pack.idx = idx or self._pack.idx
        pack.service = self._pack.service
        pack.subservice = self._pack.subservice
        pack.data = [0x10]
        self.call(pack)


# -*- coding: utf-8 -*
from mkpiano.protocols.PackData import *
class Protocol():
    _idx = 1
    _subscribe_idx = 1
    def __init__(self):
        self._engine = None
            
    def setup(self,engine):
        self._engine = engine

    def on_response(self,pack):
        self._engine.on_response(pack)

    def on_subscribe_response(self,pack):
        self._engine.on_subscribe_response(pack)  

    def check_response(self,pack):
        pass

    @property
    def next_idx(self):
        return 0

    @property
    def next_subscribe_key(self):
        self._subscribe_idx += 1
        if self._subscribe_idx>0xffff:
            self._subscribe_idx = 1
        return self._subscribe_idx
        
class NeuronProtocol(Protocol):
    def __init__(self):
        self._buffer = []

    def on_parse(self,byte):
        if byte==0xf0:
            self._engine.disable_sending()
            self._buffer = []
        # print('%#c'%byte)
        # print('%#x'%byte)
        self._buffer.append(byte)
        bufferLength = len(self._buffer)
        if bufferLength >= 2:
            if self._buffer[-1]==0xf7 and self._buffer[0]==0xf0:
                # print(self._buffer)
                self._engine.enable_sending()
                self.on_response(NeuronPackData(self._buffer))
                self._buffer = []
    
    def check_response(self,resp_pack,list_pack):
        # print(list_pack.idx,resp_pack.idx,list_pack.service,resp_pack.service,list_pack.subservice,resp_pack.subservice)
        return (list_pack.idx==resp_pack.idx) and (list_pack.service==resp_pack.service) and (list_pack.subservice==resp_pack.subservice)

class HalocodeProtocol(Protocol):
    def __init__(self):
        self._buffer = []
        self._isReceiving = False
        self._datalen = 0
        self._idx = 1
        self._ready = False
    
    def on_parse(self,byte):
        self._buffer.append(byte)
        if len(self._buffer) > 3 \
           and ((self._buffer[-1] + self._buffer[-2] + self._buffer[-4])&0xff) == self._buffer[-3] \
           and self._buffer[-4] == 0xf3:
            self._buffer = [0xf3,self._buffer[-3],self._buffer[-2],self._buffer[-1]]
            self._datalen = self._buffer[-2] + (self._buffer[-1]<<8)
            self._isReceiving = True
        if self._isReceiving:
            if self._buffer[-1]==0xf4 and self._buffer[0]==0xf3:
                self._ready = True
                pack = HalocodePackData(self._buffer)        
                if pack.type==HalocodePackData.TYPE_SCRIPT:
                    self.on_response(pack)
                elif pack.type==HalocodePackData.TYPE_SUBSCRIBE:
                    self.on_subscribe_response(pack)
                self._buffer = []
                self._datalen = 0
                self._isReceiving = False

    def on_subscribe_response(self,pack):
        data_str = "".join([ chr(i) for i in pack.data[3:len(pack.data)]])
        try:
            res = eval(data_str)
            for i in res:
                pack.subscribe_key = i
                pack.subscribe_value = res[i]
                super().on_subscribe_response(pack)
        except:
            pass

    def check_response(self,resp_pack,list_pack):
        return list_pack.idx==resp_pack.idx

    def check_subscribe_response(self,resp_pack,list_pack):
        return list_pack.subscribe_key==resp_pack.subscribe_key

    @property
    def ready(self):
        return self._ready

    @property
    def next_idx(self):
        self._idx += 1
        if self._idx>0xffff:
            self._idx = 1
        return self._idx   

class MegaPiProtocol(Protocol):
    def __init__(self):
        self._buffer = []
        self._idx = 1

    def on_parse(self,byte):
        if len(self._buffer)>0 and self._buffer[-1]==0xff and byte==0x55:
            self._buffer = [0xff]
        self._buffer.append(byte)
        bufferLength = len(self._buffer)
        if bufferLength >= 3:
            if self._buffer[-1]==0xa and self._buffer[-2]==0xd:
                self.on_response(MegaPiPackData(self._buffer))
                self._buffer = []
    
    def check_response(self,resp_pack,list_pack):
        return list_pack.idx==resp_pack.idx

    @property
    def next_idx(self):
        self._idx += 1
        if self._idx>0xff:
            self._idx = 1
        return self._idx  

class TestProtocol(Protocol):
    def __init__(self):
        self._buffer = []
        self._idx = 1

    def on_parse(self,byte):
        self._buffer.append(byte)
        bufferLength = len(self._buffer)
        if bufferLength >= 3:
            if self._buffer[-1]==0xa and self._buffer[-2]==0xd:
                self.on_response(self._buffer)
                self._buffer = []


# -*- coding: utf-8 -*
import os
from setuptools import setup
from glob import glob

dirs = ['mkpiano']
modules = ['mkpiano']
for path,dir_list,file_list in os.walk("./mkpiano"):
    for dir_name in dir_list:
        if dir_name.find("__")==-1 and dir_name.find("egg")==-1:
            dirs.append((path+"/"+dir_name).replace("/",".").replace("..",""))
    for file_name in file_list:
        if file_name.find(".py")!=-1 and file_name.find(".pyc")==-1:
            modules.append((path+"."+file_name.replace(".py","")).replace("/",".").replace("..",""))

here = os.path.dirname(__file__)
setup(
    name='mkpiano',
    version='0.2.9',
    author='makeblock',
    author_email='flashindream@gmail.com',
    url='https://makeblock.com',
    description=u'library for makeblock fingertip piano',
    packages=dirs,
    data_files=[
        (here+'/mkpiano/driver',glob('./assets/*.*')),
        (here+'/mkpiano/driver/DRVSETUP64',glob('./assets/DRVSETUP64/*.*'))
    ],
    py_modules=modules,
    install_requires=['pyserial'],
    include_package_data=True,
)


# -*- coding: utf-8 -*
import mkpiano.utils

class NeuronPackData:
    TYPE_REQUEST = 0
    TYPE_CHANGE = 1
    TYPE_PERIOD = 2
    def __init__(self,pack=[]):
        self._type = NeuronPackData.TYPE_REQUEST
        self._subscribe_key = 0
        self._subscribe_id = 0
        self._subscribe_value = 0
        self._header = 0xf0
        self._idx = 0x0
        self._service = 0x0
        self._port = 0x0
        self._slot = 0x0
        self._subservice = 0x0
        self._cmd = 0x0
        self._data = []
        self._checksum = 0x0
        self._footer = 0xf7
        self._on_response = ""
        end = len(pack)
        if(end > 0):
            self._header = pack[0]
            self._idx = pack[1]
            self._service = pack[2]
            self._subservice = pack[3]
            self._data = pack[4:end-2]
            if pack[end-2]>0:
                self._checksum = pack[end-2]
            else:
                self._checksum = self.checksum
            self._footer = pack[end-1]

    def to_buffer(self):
        bytes = bytearray()
        bytes.append(self._header)
        bytes.append(self._idx)
        bytes.append(self._service)
        bytes.append(self._subservice)
        if not self._data is None:
            _len = len(self._data)
            if _len>0:
                for i in range(_len):
                    bytes.append(self._data[i])
        bytes.append(self.checksum)
        bytes.append(self._footer)
        return bytes

    @property
    def checksum(self):
        sum = self._idx+self._service+self._subservice
        if not self._data is None:
            for i in self._data:
                sum += i
        sum &= 0x7f
        return sum

    @property
    def subscribe_id(self):
        return self._subscribe_id

    @subscribe_id.setter
    def subscribe_id(self,value):
        self._subscribe_id = value

    @property
    def subscribe_key(self):
        return self._subscribe_key

    @subscribe_key.setter
    def subscribe_key(self,value):
        self._subscribe_key = value

    @property
    def subscribe_value(self):
        return self._subscribe_value

    @subscribe_value.setter
    def subscribe_value(self,value):
        self._subscribe_value = value

    @property
    def service(self):
        return self._service

    @service.setter
    def service(self,value):
        self._service = value

    @property
    def subservice(self):
        return self._subservice

    @subservice.setter
    def subservice(self,value):
        self._subservice = value
        
    @property
    def cmd(self):
        return self._cmd

    @cmd.setter
    def cmd(self,value):
        self._cmd = value

    @property
    def idx(self):
        return self._idx

    @idx.setter
    def idx(self,value):
        self._idx = value
    
    @property
    def type(self):
        return self._type

    @type.setter
    def type(self,value):
        self._type = value

    @property
    def port(self):
        return self._port

    @port.setter
    def port(self,value):
        self._port = value

    @property
    def slot(self):
        return self._slot

    @slot.setter
    def slot(self,value):
        self._slot = value

    @property
    def data(self):
        return self._data

    @data.setter
    def data(self,value):
        self._data = value

    @property
    def on_response(self):
        return self._on_response

    @on_response.setter
    def on_response(self,value):
        self._on_response = value
    
    @staticmethod
    def broadcast():
        pack = NeuronPackData([0xf0,0xff,0x10,0x0,0xf,0xf7])
        return pack

class HalocodePackData():
    TYPE_RUN_WITHOUT_RESPONSE = 0x0
    TYPE_RUN_WITH_RESPONSE = 0x1
    TYPE_RESET = 0x2
    TYPE_SUBSCRIBE = 0x29
    TYPE_SCRIPT = 0x28
    TYPE_ONLINE = 0xd
    def __init__(self,buf=[]):
        self._type = HalocodePackData.TYPE_SCRIPT
        self._mode = HalocodePackData.TYPE_RUN_WITHOUT_RESPONSE
        self._value_name = ""
        self._header = 0xf3
        self._datalen = 0x0
        self._idx = 0
        self._type = 0x0
        self._data = []
        self._checksum = 0x0
        self._footer = 0xf4
        self._on_response = ""
        self._subscribe_key = 0
        self._subscribe_value = 0
        self._subscribe_id = 0
        self._script = ""
        end = len(buf)
        if(end > 7):
            self._header = buf[0]
            self._datalen = buf[2]+buf[3]*256
            self._type = buf[4]
            self._mode = buf[5]
            if self._type==HalocodePackData.TYPE_SCRIPT:
                self._idx = buf[6]+(buf[7]<<8)
                self._data = buf[7:end-2]
            elif self._type==HalocodePackData.TYPE_SUBSCRIBE:
                self._data = buf[5:end-2]
            elif self._type==HalocodePackData.TYPE_ONLINE:
                self._idx = buf[5]+(buf[6]<<8)
            if buf[end-2]>0:
                self._checksum = buf[end-2]
            else:
                self._checksum = self.checksum
            self._footer = buf[end-1]

    def to_buffer(self):
        bytes = bytearray()
        bytes.append(self._header)
        datalen = len(self._data)+4
        if self._type == HalocodePackData.TYPE_ONLINE:
            datalen-=1
        bytes.append((((datalen>>8)&0xff)+(datalen&0xff)+0xf3)&0xff)
        bytes.append(datalen&0xff)
        bytes.append((datalen>>8)&0xff)
        bytes.append(self._type)
        if self._type != HalocodePackData.TYPE_ONLINE:
            bytes.append(self._mode)
        bytes.append(self._idx&0xff)
        bytes.append((self._idx>>8)&0xff)
        for i in range(len(self._data)):
            bytes.append(self._data[i])
        bytes.append(self.checksum)
        bytes.append(self._footer)
        return bytes

    @property
    def checksum(self):
        sum = self._type+self._mode+((self._idx>>8)&0xff)+(self._idx&0xff)
        for i in range(len(self._data)):
            sum += self._data[i]
        return sum&0xff

    @property
    def script(self):
        return self._script

    @script.setter
    def script(self,value):
        self._script = value

        script_data = bytearray(value, "utf8")

        datalen = len(script_data)
        self._data = [datalen&0xff,(datalen>>8)&0xff]
        self._data.extend(script_data)

    @property
    def subscribe_key(self):
        return self._subscribe_key

    @subscribe_key.setter
    def subscribe_key(self,value):
        self.script = self.script.format(value)
        self._subscribe_key = value

    @property
    def subscribe_value(self):
        return self._subscribe_value

    @subscribe_value.setter
    def subscribe_value(self,value):
        self._subscribe_value = value

    @property
    def value_name(self):
        return self._value_name

    @value_name.setter
    def value_name(self,value):
        self._value_name = value

    @property
    def subscribe_id(self):
        return self._subscribe_id

    @subscribe_id.setter
    def subscribe_id(self,value):
        self._subscribe_id = value

    @property
    def type(self):
        return self._type

    @type.setter
    def type(self,value):
        self._type = value

    @property
    def mode(self):
        return self._mode

    @mode.setter
    def mode(self,value):
        self._mode = value

    @property
    def datalen(self):
        return self._datalen

    @datalen.setter
    def datalen(self,value):
        self._datalen = value

    @property
    def idx(self):
        return self._idx

    @idx.setter
    def idx(self,value):
        self._idx = value

    @property
    def data(self):
        return self._data

    @data.setter
    def data(self,value):
        self._data = value

    @property
    def on_response(self):
        return self._on_response

    @on_response.setter
    def on_response(self,value):
        self._on_response = value
    
    @staticmethod
    def broadcast():
        return HalocodePackData([0xf3, 0xf6, 0x03, 0x0, 0x0d, 0x0, 0x01, 0x0e, 0xf4])

    @staticmethod
    def repl_mode():
        pack =  HalocodePackData()
        pack.type = HalocodePackData.TYPE_SCRIPT
        pack.mode = HalocodePackData.TYPE_RUN_WITH_RESPONSE
        pack.script = "global_objects.communication_o.enable_protocol(global_objects.communication_o.REPL_PROTOCOL_GROUP_ID)"
        return pack

class MegaPiPackData():
    ACTION_GET = 1
    ACTION_RUN = 2
    ACTION_RESET = 4
    ACTION_START = 5
    def __init__(self,pack=[]):
        self._headers = [0xff,0x55]
        self._idx = 0x0
        self._action = MegaPiPackData.ACTION_RUN
        self._port = 0
        self._slot = 0
        self._module = 0
        self._subscribe_id = 0
        self._data = []
        self._checksum = 0x0
        self._on_response = ""
        self._footer = 0xa
        end = len(pack)
        if(end > 0):
            self._headers = pack[0:2]
            self._idx = pack[2]
            self._data = pack[3:end-2]
            self._footer = pack[end-1]

    def to_buffer(self):
        bytes = bytearray()
        bytes.extend(self._headers)
        bytes.append(0)
        bytes.append(self._idx)
        bytes.append(self._action)
        bytes.append(self._module)
        for i in range(len(self._data)):
            bytes.append(self._data[i])
        bytes.append(self._footer)
        bytes[2] = len(bytes)-3
        return bytes

    @property
    def checksum(self):
        sum = self._idx+self._action+self._module
        for i in range(len(self._data)):
            sum += self._data[i]
        sum &= 0x7f
        return sum

    @property
    def module(self):
        return self._module

    @module.setter
    def module(self,value):
        self._module = value

    @property
    def idx(self):
        return self._idx

    @idx.setter
    def idx(self,value):
        self._idx = value
    
    @property
    def port(self):
        return self._port

    @port.setter
    def port(self,value):
        self._port = value

    @property
    def slot(self):
        return self._slot

    @slot.setter
    def slot(self,value):
        self._slot = value
        
    @property
    def action(self):
        return self._action

    @action.setter
    def action(self,value):
        self._action = value

    @property
    def data(self):
        return self._data

    @data.setter
    def data(self,value):
        self._data = value

    @property
    def subscribe_id(self):
        return self._subscribe_id

    @subscribe_id.setter
    def subscribe_id(self,value):
        self._subscribe_id = value

    @property
    def on_response(self):
        return self._on_response

    @on_response.setter
    def on_response(self,value):
        self._on_response = value
    


# -*- coding: utf-8 -*
from mkpiano.modules.mbuild import *
import mkpiano.protocols as Protocols
from ...boards.base import _BaseBoard
from ...protocols.PackData import NeuronPackData
from ...comm.SerialPort import SerialPort
from time import sleep
MODE_REQUEST = 0
MODE_CHANGE = 1
MODE_PERIOD = 2
GESTURE_NONE = 0
GESTURE_WAVING = 1
GESTURE_MOVING_UP = 2
GESTURE_MOVING_DOWN = 3
C3 = 131
D3 = 147
E3 = 165
F3 = 174
G3 = 196
A3 = 220
B3 = 247
C4 = 261
D4 = 293
E4 = 329
F4 = 349
G4 = 392
A4 = 440
B4 = 493
C5 = 523
D5 = 587
E5 = 659
F5 = 698
G5 = 784
A5 = 880
B5 = 987
def connect(device=None,channel=None):
    return __mBuild(device or channel)

create = connect

class __mBuild(_BaseBoard):
    _device = None
    def __init__(self,device=None):
        self._type = _BaseBoard.mBuild
        if not device is None:
            self._device = device
            super().__init__(_BaseBoard.mBuild,device)
            self.broadcast()

    def broadcast(self):
        self.call(NeuronPackData.broadcast())

    _speaker = {}
    def Speaker(self,idx=1):
        if not idx in self._speaker:
            self._speaker[idx] = Speaker(self,idx)
        return self._speaker[idx]

    @property
    def speaker(self):
        return self.Speaker()

    _ledmatrix = {}
    def LedMatrix(self,idx=1):
        if not idx in self._ledmatrix:
            self._ledmatrix[idx] = LedMatrix(self,idx)
        return self._ledmatrix[idx]

    @property
    def ledmatrix(self):
        return self.LedMatrix(self)

    _rgbled = {}
    def RGBLed(self,idx=1):
        if not idx in self._rgbled:
            self._rgbled[idx] = RGBLed(self,idx)
        return self._rgbled[idx]

    @property
    def rgbled(self):
        return self.RGBLed(self)

    def LedStrip(self,idx=1):
        return LedStrip(self,idx)

    _piano = {}
    def Piano(self,idx=1):
        if not idx in self._piano:
            self._piano[idx] = FingertipPiano(self,idx)
        return self._piano[idx]

    @property
    def piano(self):
        return self.Piano()

    _ultrasonic = {}
    def Ultrasonic(self,idx=1):
        if not idx in self._ultrasonic:
            self._ultrasonic[idx] = Ultrasonic(self,idx)
        return self._ultrasonic[idx]

    @property
    def ultrasonic(self):
        return self.Ultrasonic()

    _button = {}
    def Button(self,idx=1):
        if not idx in self._button:
            self._button[idx] = Button(self,idx)
        return self._button[idx]

    @property
    def button(self):
        return self.Button()

    _slider = {}
    def Slider(self,idx=1):
        if not idx in self._slider:
            self._slider[idx] = Slider(self,idx)
        return self._slider[idx]

    @property
    def slider(self):
        return self.Slider()

    _joystick = {}
    def Joystick(self,idx=1):
        if not idx in self._joystick:
            self._joystick[idx] = Joystick(self,idx)
        return self._joystick[idx]
        
    @property
    def joystick(self):
        return self.Joystick()

    _soilmoisture = {}
    def SoilMoisture(self,idx=1):
        if not idx in self._soilmoisture:
            self._soilmoisture[idx] = SoilMoisture(self,idx)
        return self._soilmoisture[idx]

    @property
    def soilmoisture(self):
        return self.SoilMoisture()

    _laserranging = {}
    def LaserRanging(self,idx=1):
        if not idx in self._laserranging:
            self._laserranging[idx] = LaserRanging(self,idx)
        return self._laserranging[idx]

    @property
    def laserranging(self):
        return self.LaserRanging()

    _flame = {}
    def Flame(self,idx=1):
        if not idx in self._flame:
            self._flame[idx] = Flame(self,idx)
        return self._flame[idx]

    @property
    def flame(self):
        return self.Flame()

    _touch = {}
    def Touch(self,idx=1):
        if not idx in self._touch:
            self._touch[idx] = Touch(self,idx)
        return self._touch[idx]

    @property
    def touch(self):
        return self.Touch()

    _sound = {}
    def Sound(self,idx=1):
        if not idx in self._sound:
            self._sound[idx] = Sound(self,idx)
        return self._sound[idx]

    @property
    def sound(self):
        return self.Sound()

    _light = {}
    def Light(self,idx=1):
        if not idx in self._light:
            self._light[idx] = Light(self,idx)
        return self._light[idx]

    @property
    def light(self):
        return self.Light()
    
    _pirmotion = {}
    def PIRMotion(self,idx=1):
        if not idx in self._pirmotion:
            self._pirmotion[idx] = PIRMotion(self,idx)
        return self._pirmotion[idx]

    @property
    def pirmotion(self):
        return self.PIRMotion()

    _magnetic = {}
    def Magnetic(self,idx=1):
        if not idx in self._magnetic:
            self._magnetic[idx] = Magnetic(self,idx)
        return self._magnetic[idx]

    @property
    def magnetic(self):
        return self.Magnetic()

    _angle = {}
    def Angle(self,idx=1):
        if not idx in self._angle:
            self._angle[idx] = Angle(self,idx)
        return self._angle[idx]

    @property
    def angle(self):
        return self.Angle()

    _motion = {}
    def Motion(self,idx=1):
        if not idx in self._motion:
            self._motion[idx] = Motion(self,idx)
        return self._motion[idx]

    @property
    def motion(self):
        return self.Motion()

    _servo = {}
    def Servo(self,idx=1):
        if not idx in self._servo:
            self._servo[idx] = Servo(self,idx)
        return self._servo[idx]

    @property
    def servo(self):
        return self.Servo()

    _dcmotor = {}
    def DCMotor(self,idx=1):
        if not idx in self._dcmotor:
            self._dcmotor[idx] = DCMotor(self,idx)
        return self._dcmotor[idx]

    @property
    def dcmotor(self):
        return self.DCMotor()

    _encodermotor = {}
    def EncoderMotor(self,idx=1):
        if not idx in self._encodermotor:
            self._encodermotor[idx] = EncoderMotor(self,idx)
        return self._encodermotor[idx]

    @property
    def encodermotor(self):
        return self.EncoderMotor()

    _color = {}
    def Color(self,idx=1):
        if not idx in self._color:
            self._color[idx] = Color(self,idx)
        return self._color[idx]

    @property
    def color(self):
        return self.Color()

    _gpio = {}
    def GPIO(self,idx=1):
        if not idx in self._gpio:
            self._gpio[idx] = GPIO(self,idx)
        return self._gpio[idx]

    @property
    def gpio(self):
        return self.GPIO()

    _powermanager = {}
    def PowerManager(self,idx=1):
        if not idx in self._powermanager:
            self._powermanager[idx] = PowerManager(self,idx)
        return self._powermanager[idx]

    @property
    def powermanager(self):
        return self.PowerManager()

    _infrarer = {}
    def Infrarer(self,idx=1):
        if not idx in self._infrarer:
            self._infrarer[idx] = Infrarer(self,idx)
        return self._infrarer[idx]

    @property
    def infrarer(self):
        return self.Infrarer()

    _temp = {}
    def Temperature(self,idx=1):
        if not idx in self._temp:
            self._temp[idx] = Temperature(self,idx)
        return self._temp[idx]

    @property
    def temperature(self):
        return self.Temperature()

    _humiture = {}
    def Humiture(self,idx=1):
        if not idx in self._humiture:
            self._humiture[idx] = Humiture(self,idx)
        return self._humiture[idx]

    @property
    def humiture(self):
        return self.Humiture()

    _extdcmotor = {}
    def ExtDCMotor(self,idx=1):
        if not idx in self._extdcmotor:
            self._extdcmotor[idx] = ExtDCMotor(self,idx)
        return self._extdcmotor[idx]

    @property
    def extdcmotor(self):
        return self.ExtDCMotor()

    _smartservo = {}
    def SmartServo(self,idx=1):
        if not idx in self._smartservo:
            self._smartservo[idx] = SmartServo(self,idx)
        return self._smartservo[idx]

    @property
    def smartservo(self):
        return self.SmartServo()
    
    